<?php

if ( ! class_exists( 'Meta_Domainer_Import_Export' ) ) {

    /**
     * Import and export functionality
     * 
     * @since 1.0.4
     */
    class Meta_Domainer_Import_Export {

        /**
         * Constructor
         * 
         * @since 1.0.4
         */
        function __construct() {

            // add the admin page
            add_action( 'admin_menu', array( $this, 'add_admin_page' ) );

            // process export
            add_action( 'admin_init', array( $this, 'process_export' ) );

            // process import
            add_action( 'admin_init', array( $this, 'process_import' ) );

            // allow CSV uploads
            add_filter( 'upload_mimes', array( $this, 'allow_csv_uploads' ) );

        }

        /**
         * Add the admin page
         * 
         * @since 1.0.4
         */
        function add_admin_page() {

            $parent = 'edit.php?post_type=meta_domainer';
            $page_title = __( 'Import/Export', 'meta-domainer-import-export' );
            $menu_title = __( 'Import/Export', 'meta-domainer-import-export' );
            $capability = 'manage_options';
            $slug = 'meta_domainer_import_export';
            $function = array( $this, 'display_admin_page' );

            add_submenu_page( 
                $parent, 
                $page_title, 
                $menu_title, 
                $capability, 
                $slug, 
                $function 
            );

        }

        /**
         * Display the admin page
         * 
         * @since 1.0.0
         */
        function display_admin_page() {

            $export_url = add_query_arg( array(
				'meta_domainer_export' => 'csv',
			), admin_url() );

            ?>
            <style type="text/css">
                .meta-domainer-admin-options {
                    padding: 20px;
                    background: #fff;
                    border: 1px solid rgba( 0, 0, 0, 0.1 );
                }
                .meta-domainer-admin-options h2 {
                    display: none;
                }
                .meta-domainer-admin-options .form-table tr {
                    border-bottom: 1px solid rgba( 0, 0, 0, 0.1 );
                }
                    .meta-domainer-admin-options .form-table td,
                    .meta-domainer-admin-options .form-table th {
                        padding: 30px 0;	
                    }
                .meta-domainer-admin-options input[type="text"] {
                    width: 300px;
                }
                .meta-domainer-admin-options input[type="checkbox"] {
                    vertical-align: middle;
                }
                .meta-domainer-admin-options p.description {
                    max-width: 400px;
                }
                .meta-domainer-admin-options-success {
                    color: green;
                    font-weight: bold;
                }
            </style>
            <div class="wrap">
                
                <div class="meta-domainer-admin-options">

                    <h1><?php esc_html_e( 'Export/Import Function - See Documentation', 'meta-domainer-import-export' ); ?></h1>
                    <p><?php esc_html_e( 'This is where you can export and import domains. The CSV Export is great for Bulk Data Export. For easier import, please see our documentation for sample CSV..', 'meta-domainer-import-export' ); ?></p>
					<p><a target="blank" href="https://account.metadomainer.com/documentation/">Documentation</a></p>

                    <table class="form-table">
                        <tr>
                            <th><?php esc_html_e( 'Export', 'meta-domainer-import-export' ); ?></th>
                            <td><a href="<?php echo esc_url( $export_url ); ?>" class="button button-primary"><?php esc_html_e( 'Download CSV export of ALL Data', 'meta-domainer-import-export' ); ?></a></td>
                        </tr>
                        <tr>
                            <th><?php esc_html_e( 'Import', 'meta-domainer-import-export' ); ?></th>
                            <td>
                                <form action="" method="post" enctype="multipart/form-data">
                                    <input type="file" name="meta_domainer_file"><br>
                                    <input type="submit" class="button button-primary" value="<?php esc_attr_e( 'Import Domains', 'meta-domainer-import-export' ); ?>">
                                    <input type="hidden" name="meta_domainer_import" value="csv">
                                </form>
                                <?php
                                    global $meta_domainer_import_message;
                                    if ( ! empty( $meta_domainer_import_message ) ) {
                                        ?><p class="meta-domainer-admin-options-success"><?php echo $meta_domainer_import_message; ?></p><?php
                                    }
                                ?>
                            </td>
                        </tr>
                    </table>

                </div><!-- .meta-domainer-admin-options -->

            </div>
            <?php

        }

        /**
         * Generate export
         * 
         * @since 1.0.4
         */
        function process_export() {

            // not exporting, return
            if ( empty( $_GET['meta_domainer_export'] ) || $_GET['meta_domainer_export'] !== 'csv' ) {
                return;                
            }

            // not administrator, return
            if ( ! current_user_can( 'manage_options' ) ) {
                return;
            }

            // will hold data for CSV
            $data = array();

            // query arguments
            $args = array(
                'post_type' => 'meta_domainer',
                'posts_per_page' => -1,
                'no_found_rows' => true,
                'order' => 'ASC',
            );

            // do the query
            $meta_domainer_query = new WP_Query( $args );

            // if posts found
            if ( $meta_domainer_query->have_posts() ) {

                // loop through posts
                while ( $meta_domainer_query->have_posts() ) {
                    
                    // set up post data
                    $meta_domainer_query->the_post();                
                    
                    // get local values
                    $local = meta_domainer_get_local_settings();

                    // get categories
                    $categories = wp_get_post_terms( get_the_ID(), 'meta_domainer_cats', array( 'fields' => 'names' ) );
                    if ( is_array( $categories ) ) {
                        $categories = implode( ',', $categories );
                    }

                    // get tags
                    $tags = wp_get_post_terms( get_the_ID(), 'meta_domainer_tags', array( 'fields' => 'names' ) );
                    if ( is_array( $tags ) ) {
                        $tags = implode( ',', $tags );
                    }

                    // populate data array
                    $data[] = array(
                        
                        'title'      => get_the_title(),
                        'content'    => get_the_content(),
                        'categories' => $categories,
                        'tags'       => $tags,
                        
                        'template'         => $local['template'],
                        
                        'company_name'     => $local['company_name'],
                        'copyright_notice' => $local['copyright_notice'],
                        'affiliate_notice' => $local['affiliate_notice'],
                        'price'            => $local['price'],
                        'buy_url'          => $local['buy_url'],
                        'contact_url'      => $local['contact_url'],
                        'privacy_url'      => $local['privacy_url'],
                        'domain_url'       => $local['domain_url'],
                        'custom_menu'      => $local['custom_menu'],
                        
                        'disable_view_counter'   => $local['disable_view_counter'],
                        'disable_cookie_consent' => $local['disable_cookie_consent'],
                        'disable_ad_code'        => $local['disable_ad_code'],
                        'enable_js'              => $local['enable_js'],

                        'amazon_placement'           => $local['amazon_placement'],
                        'amazon_search_bar'          => $local['amazon_search_bar'],
                        'amazon_search_bar_position' => $local['amazon_search_bar_position'],
                        'amazon_tracking_id'         => $local['amazon_tracking_id'],
                        'amazon_ad_mode'             => $local['amazon_ad_mode'],
                        'amazon_ad_type'             => $local['amazon_ad_type'],
                        'amazon_marketplace'         => $local['amazon_marketplace'],
                        'amazon_region'              => $local['amazon_region'],
                        'amazon_title'               => $local['amazon_title'],
                        'amazon_search_phrase'       => $local['amazon_search_phrase'],
                        'amazon_category'            => $local['amazon_category'],
                        'amazon_linkid'              => $local['amazon_linkid'],
                        
                        'seo_title'       => $local['seo_title'],
                        'seo_description' => $local['seo_description'],
                        
                        'header_code'    => $local['header_code'],
                        'custom_css'     => $local['custom_css'],
                        'css_class'      => $local['css_class'],
                        'footer_code'    => $local['footer_code'],

                        'redirect_url'     => $local['redirect_url'],
                        'redirect_refresh' => $local['redirect_refresh'],
                        
                    );

                }

                // reset
                wp_reset_postdata();

            }

            // php memory
            $fh = fopen('php://temp', 'rw');
            
            // headers
            fputcsv($fh, array_keys(current($data)));

            // data
            foreach ( $data as $row ) {
                fputcsv($fh, $row);
            }

            // finish
            rewind($fh);
            $csv = stream_get_contents($fh);
            fclose($fh);

            // start the download and exit
 			nocache_headers();
 			header( 'Content-Type: text/csv' );
			header( 'Content-Disposition: attachment; filename="export.csv"' );
 			echo $csv;
			exit;

        }

        /**
         * Process import
         * 
         * @since 1.0.4
         */
        function process_import() {

             // not importing, return
             if ( empty( $_POST['meta_domainer_import'] ) || $_POST['meta_domainer_import'] !== 'csv' ) {
                return;                
            }

            // not administrator, return
            if ( ! current_user_can( 'manage_options' ) ) {
                return;
            }

            // load filesystem if not already loaded
            if ( ! function_exists( 'wp_handle_upload' ) ) {
                require_once( ABSPATH . 'wp-admin/includes/file.php' );
            }
            
            // get uploaded file
            $uploadedfile = $_FILES['meta_domainer_file'];
            
            $upload_overrides = array( 'test_form' => false );
            
            // handle upload
            $movefile = wp_handle_upload( $uploadedfile, $upload_overrides );
            
            if ( $movefile && ! isset( $movefile['error'] ) ) {        

                $import_data = new ParseCsv\Csv($movefile['file']);              
                if ( ! empty( $import_data->data ) ) {
                    $import_data = $import_data->data;
                }

                if ( is_array( $import_data ) ) {

                    $count = 0;

                    foreach ( $import_data as $domain ) {

                        $count++;

                        // get general
                        $domain_title = $domain['title'];
                        $domain_content = $domain['content'];
                        $domain_cats = $domain['categories'];
                        $domain_tags = $domain['tags'];

                        // remove general ( rest is meta )
                        unset( $domain['title'] );
                        unset( $domain['content'] );
                        unset( $domain['categories'] );
                        unset( $domain['tags'] );

                        // prefix
                        $domain = array_combine(
                            array_map(function($k){ return '_meta_domainer_'.$k; }, array_keys($domain)),
                            $domain
                        );

                        $args = array(
                            'post_type' => 'meta_domainer',
                            'post_title' => $domain_title,
                            'post_content' => $domain_content,
                            'post_status' => 'publish',
                            'meta_input' => $domain
                        );

                        // get cats and tags
                        if ( ! empty( $domain_cats ) || ! empty( $domain_tags ) ) {

                            $args['tax_input'] = array();

                            // tags
                            if ( ! empty( $domain_tags ) ) {
                                $args['tax_input']['meta_domainer_tags'] = $domain_tags;
                            }

                            // cats ( can't use names like for the tags, needs IDs)
                            if ( ! empty( $domain_cats ) ) {

                                // turn into aray
                                $cats = $domain_cats;
                                $cats_ids = array();
                                $cats = explode( ',', $cats );

                                // loop through cats
                                foreach ( $cats as $cat ) {
                                    
                                    // exists, get ID
                                    if ( $cat_object = get_term_by( 'name', $cat, 'meta_domainer_cats' ) ) {
                                        $cats_ids[] = $cat_object->term_id;

                                    // does not exist, create and get ID
                                    } else {
                                        $new_cat = wp_insert_term( $cat, 'meta_domainer_cats' );
                                        if ( ! empty( $new_cat['term_id'] ) ) {
                                            $cats_ids[] = $new_cat['term_id'];
                                        }
                                    }

                                }

                                // set tax input for post insert
                                $args['tax_input']['meta_domainer_cats'] = $cats_ids;

                            }

                        }

                        // insert post
                        wp_insert_post( $args );

                    }

                    global $meta_domainer_import_message;
                    $meta_domainer_import_message = $count . ' domains succesfuly imported';

                }
                
                
            } else {
                var_dump( $movefile['error'] );
                exit();
            }

        }

        /**
         * Allow CSV uploads
         * 
         * @since 1.0.4
         */
        function allow_csv_uploads( $mimes ) {

            $mimes['csv'] = 'text/csv';

            return $mimes;

        }

    }

}

new Meta_Domainer_Import_Export();